#!/bin/sh
#
# Debian-specific package management functions.
#
# Copyright 2023 Andrew Wood
#
# License GPLv3+: GNU GPL version 3 or later; see `docs/COPYING'.
#

# Setup required for all Debian package actions - set "aptFlag" and
# DEBIAN_FRONTEND according to whether interactive mode should be used; and
# set "aptArgs" to the sanitised arguments.
#
actDebianPackage () {
	aptFlag=""
	aptArgs=""

	# Set the arguments and environment according to whether interactive
	# mode is enabled.
	DEBIAN_FRONTEND=""
	test "${optionNonInteractive}" -eq 1 && aptFlag="-y"
	test "${optionNonInteractive}" -eq 1 && DEBIAN_FRONTEND="noninteractive"
	export DEBIAN_FRONTEND

	# Sanitise the input arguments.
	aptArgs="$(actSanitisePackageList "$*")"
}

# Install packages $*.
actDebianPackageInstall () {
	actRun apt-get --quiet update >/dev/null
	actRun apt-get ${aptFlag} --no-upgrade install ${aptArgs}
	cmdExitStatus=$?
	actDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Upgrade packages $*.
actDebianPackageUpgrade () {
	actRun apt-get --quiet update >/dev/null
	actRun apt-get ${aptFlag} --only-upgrade install ${aptArgs}
	cmdExitStatus=$?
	actDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Downgrade packages $*.
actDebianPackageDowngrade () {
	actRun apt-get --quiet update >/dev/null
	actRun apt-get ${aptFlag} --allow-downgrades install ${aptArgs}
	cmdExitStatus=$?
	actDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Remove packages $*.
actDebianPackageRemove () {
	actRun apt-get ${aptFlag} remove ${aptArgs}
	cmdExitStatus=$?
	actDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Reinstall packages $*.
actDebianPackageReinstall () {
	actRun apt-get --quiet update >/dev/null
	actRun apt-get ${aptFlag} --reinstall install ${aptArgs}
	cmdExitStatus=$?
	actDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Output information about packages $*, one per line, TAB separate columns -
# package, architecture, version, install epoch time, vendor, type, summary.
#
actDebianPackageInfo () {
	# Include the status in the query so we can check the package is
	# installed (second character is "i") and skip those that are not. 
	# Also, use \r as a field separator since it's unlikely to appear in
	# the name or synopsis, so we can strip tabs from inside columns.
	actRun dpkg-query --show --showformat='${db:Status-Abbrev}\r${Package}\r${Architecture}\r${Version}\r${db-fsys:Last-Modified}\r${Maintainer}\rdeb\r${binary:Synopsis}\n' ${aptArgs} 2>/dev/null \
	| sed -n 's/^.i.\r//p' \
	| tr '\t' ' ' \
	| tr '\r' '\t'
	return 0
}

# List the files owned by package $1.
actDebianPackageFiles () {
	actRun dpkg-query --listfiles ${aptArgs}
	cmdExitStatus=$?
	actDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# List the package which owns file $1.
actDebianPackageOwningFile () {
	queryOutput=$(actRun dpkg-query --search "$1")
	cmdExitStatus=$?
	actDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 || return 1
	printf "%s\n" "${queryOutput}" | cut -d : -f 1 | tr ',' '\n' | sed 's,^ *,,' | sort -u
	return 0
}

# Refresh the cache.
actDebianPackageCacheUpdate () {
	actRun apt-get update
	cmdExitStatus=$?
	actDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Clean all caches.
actDebianPackageCacheClean () {
	actRun apt-get clean
	cmdExitStatus=$?
	actDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# List packages ready to upgrade.
actDebianPackageUpgradeList () {
	actRun apt-get --quiet update >/dev/null
	queryOutput=$(actRun apt-get upgrade --simulate)
	cmdExitStatus=$?
	actDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 || return 1
	printf "%s\n" "${queryOutput}" | sed -n '/^Inst /{s///;s/ .*$//;p}' | sort -u
	return 0
}

# Upgrade all packages.
actDebianPackageUpgradeAll () {
	actRun apt-get --quiet update >/dev/null
	actRun apt-get ${aptFlag} upgrade
	cmdExitStatus=$?
	actDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# List all installed packages.
actDebianPackageListInstalled () {
	actDebianPackageInfo
	return $?
}
